package jpcap.packet;

import java.io.UnsupportedEncodingException;

public class TextPacket extends Packet {
  private static final long serialVersionUID = 6199417211903927904L;

  private static final String UNKNOWN_ADDRESS = "Unknown";
  
	public String getText() {
		if (data == null)
			return null;
		
		try {
			return new String(data, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return null;
		}
	}
	
  public String getReceiver() {
    if (datalink instanceof TTYPacket == false) {
      return UNKNOWN_ADDRESS;
    }

    TTYPacket ttyPacket = (TTYPacket) datalink;

    if (ttyPacket.direction == TTYPacket.TERMINAL_TO_SERVER) {
      return ttyPacket.connectionParameters;
    }

    return buildTermAddress(ttyPacket);
  }

  public String getSender() {
    if (datalink instanceof TTYPacket == false) {
      return UNKNOWN_ADDRESS;
    }

    TTYPacket ttyPacket = (TTYPacket) datalink;

    if (ttyPacket.direction == TTYPacket.TERMINAL_TO_SERVER) {
      return buildTermAddress(ttyPacket);
    }

    return ttyPacket.connectionParameters;
  }

  private String buildTermAddress(TTYPacket ttyPacket) {
    switch (ttyPacket.connectionType) {
      case TTYPacket.SSH_CONNECTION:
        return "sshterm";
        
      case TTYPacket.LOCAL_CONNECTION:
        return "localterm";
        
      case TTYPacket.TELNET_CONNECTION:
        return "telnetterm";
    }
  
    return "";
  }
}
